@@ -118,33 +118,7 @@ void signal_div(const spx_sig_t *x, spx_sig_t *y, spx_word32_t scale, int len)
 
 #ifdef FIXED_POINT
 
-int normalize16(const spx_sig_t *x, spx_word16_t *y, int max_scale, int len)
-{
-   int i;
-   spx_sig_t max_val=1;
-   int sig_shift;
-   
-   for (i=0;i<len;i++)
-   {
-      spx_sig_t tmp = x[i];
-      if (tmp<0)
-         tmp = -tmp;
-      if (tmp >= max_val)
-         max_val = tmp;
-   }
 
-   sig_shift=0;
-   while (max_val>max_scale)
-   {
-      sig_shift++;
-      max_val >>= 1;
-   }
-
-   for (i=0;i<len;i++)
-      y[i] = SHR(x[i], sig_shift);
-   
-   return sig_shift;
-}
 
 spx_word16_t compute_rms(const spx_sig_t *x, int len)
 {
@@ -187,6 +161,38 @@ spx_word16_t compute_rms(const spx_sig_t *x, int len)
    return SHR(SHL((spx_word32_t)spx_sqrt(1+DIV32(sum,len)),(sig_shift+3)),SIG_SHIFT);
 }
 
+#if defined(ARM4_ASM) || defined(ARM5E_ASM)
+#include "filters_arm4.h"
+#else
+
+
+int normalize16(const spx_sig_t *x, spx_word16_t *y, int max_scale, int len)
+{
+   int i;
+   spx_sig_t max_val=1;
+   int sig_shift;
+   
+   for (i=0;i<len;i++)
+   {
+      spx_sig_t tmp = x[i];
+      if (tmp<0)
+         tmp = -tmp;
+      if (tmp >= max_val)
+         max_val = tmp;
+   }
+
+   sig_shift=0;
+   while (max_val>max_scale)
+   {
+      sig_shift++;
+      max_val >>= 1;
+   }
+
+   for (i=0;i<len;i++)
+      y[i] = SHR(x[i], sig_shift);
+   
+   return sig_shift;
+}
 
 void filter_mem2(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
@@ -230,6 +236,9 @@ void iir_mem2(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, in
    }
 }
 
+#endif
+
+
 
 void fir_mem2(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
@@ -469,7 +478,7 @@ CombFilterMem *mem
 {
    int i;
    spx_word16_t exc_energy=0, new_exc_energy=0;
-   float gain;
+   spx_word16_t gain;
    spx_word16_t step;
    spx_word16_t fact;
 
@@ -479,6 +488,15 @@ CombFilterMem *mem
      exc_energy+=((float)exc[i])*exc[i];*/
 
    /*Some gain adjustment if pitch is too high or if unvoiced*/
+#ifdef FIXED_POINT
+   {
+      spx_word16_t g = gain_3tap_to_1tap(pitch_gain)+gain_3tap_to_1tap(mem->last_pitch_gain);
+      if (g > 166)
+         comb_gain = MULT16_16_Q15(DIV32_16(SHL(165,15),g), comb_gain);
+      if (g < 64)
+         comb_gain = MULT16_16_Q15(SHL(g, 9), comb_gain);
+   }
+#else
    {
       float g=0;
       g = GAIN_SCALING_1*.5*(gain_3tap_to_1tap(pitch_gain)+gain_3tap_to_1tap(mem->last_pitch_gain));
@@ -487,6 +505,7 @@ CombFilterMem *mem
       if (g<.5)
          comb_gain*=2.*g;
    }
+#endif
    step = DIV32(COMB_STEP, nsf);
    fact=0;
 
@@ -515,22 +534,25 @@ CombFilterMem *mem
    /*Amplitude after enhancement*/
    new_exc_energy = compute_rms(new_exc, nsf);
 
-   /*Compute scaling factor and normalize energy*/
-   gain = (exc_energy)/(.1+new_exc_energy);
-   if (gain < .5)
-      gain=.5;
-   if (gain>.9999)
-      gain=.9999;
+   if (exc_energy > new_exc_energy)
+      exc_energy = new_exc_energy;
+   
+   gain = DIV32_16(SHL(exc_energy,15),1+new_exc_energy);
 
 #ifdef FIXED_POINT
+   if (gain < 16384)
+      gain = 16384;
+#else
+   if (gain < .5)
+      gain=.5;
+#endif
+
+#ifdef FIXED_POINT
+   for (i=0;i<nsf;i++)
    {
-      spx_word16_t gg = gain*32768;
-      for (i=0;i<nsf;i++)
-   {
-      mem->smooth_gain = MULT16_16_Q15(31457,mem->smooth_gain) + MULT16_16_Q15(1311,gg);
+      mem->smooth_gain = MULT16_16_Q15(31457,mem->smooth_gain) + MULT16_16_Q15(1311,gain);
       new_exc[i] = MULT16_32_Q15(mem->smooth_gain, new_exc[i]);
    }
-   }
 #else
    for (i=0;i<nsf;i++)
    {
