@@ -41,11 +41,17 @@
 #include <speex/speex_bits.h>
 #include "math_approx.h"
 
-#include <stdio.h>
+#ifndef NULL
+#define NULL 0
+#endif
+
 
 #ifdef _USE_SSE
 #include "ltp_sse.h"
+#elif defined (ARM4_ASM) || defined(ARM5E_ASM)
+#include "ltp_arm4.h"
 #else
+
 static spx_word32_t inner_prod(const spx_word16_t *x, const spx_word16_t *y, int len)
 {
    int i;
@@ -62,6 +68,70 @@ static spx_word32_t inner_prod(const spx_word16_t *x, const spx_word16_t *y, int
    return sum;
 }
 
+#if 0 /* Enable this for machines with enough registers (i.e. not x86) */
+static void pitch_xcorr(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
+{
+   int i,j;
+   for (i=0;i<nb_pitch;i+=4)
+   {
+      /* Compute correlation*/
+      //corr[nb_pitch-1-i]=inner_prod(x, _y+i, len);
+      spx_word32_t sum1=0;
+      spx_word32_t sum2=0;
+      spx_word32_t sum3=0;
+      spx_word32_t sum4=0;
+      const spx_word16_t *y = _y+i;
+      const spx_word16_t *x = _x;
+      spx_word16_t y0, y1, y2, y3;
+      //y0=y[0];y1=y[1];y2=y[2];y3=y[3];
+      y0=*y++;
+      y1=*y++;
+      y2=*y++;
+      y3=*y++;
+      for (j=0;j<len;j+=4)
+      {
+         spx_word32_t part1;
+         spx_word32_t part2;
+         spx_word32_t part3;
+         spx_word32_t part4;
+         part1 = MULT16_16(*x,y0);
+         part2 = MULT16_16(*x,y1);
+         part3 = MULT16_16(*x,y2);
+         part4 = MULT16_16(*x,y3);
+         x++;
+         y0=*y++;
+         part1 = MAC16_16(part1,*x,y1);
+         part2 = MAC16_16(part2,*x,y2);
+         part3 = MAC16_16(part3,*x,y3);
+         part4 = MAC16_16(part4,*x,y0);
+         x++;
+         y1=*y++;
+         part1 = MAC16_16(part1,*x,y2);
+         part2 = MAC16_16(part2,*x,y3);
+         part3 = MAC16_16(part3,*x,y0);
+         part4 = MAC16_16(part4,*x,y1);
+         x++;
+         y2=*y++;
+         part1 = MAC16_16(part1,*x,y3);
+         part2 = MAC16_16(part2,*x,y0);
+         part3 = MAC16_16(part3,*x,y1);
+         part4 = MAC16_16(part4,*x,y2);
+         x++;
+         y3=*y++;
+         
+         sum1 = ADD32(sum1,SHR(part1,6));
+         sum2 = ADD32(sum2,SHR(part2,6));
+         sum3 = ADD32(sum3,SHR(part3,6));
+         sum4 = ADD32(sum4,SHR(part4,6));
+      }
+      corr[nb_pitch-1-i]=sum1;
+      corr[nb_pitch-2-i]=sum2;
+      corr[nb_pitch-3-i]=sum3;
+      corr[nb_pitch-4-i]=sum4;
+   }
+
+}
+#else
 static void pitch_xcorr(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
 {
    int i;
@@ -72,6 +142,9 @@ static void pitch_xcorr(const spx_word16_t *_x, const spx_word16_t *_y, spx_word
    }
 
 }
+#endif
+
+
 
 #endif
 
@@ -100,7 +173,6 @@ void open_loop_nbest_pitch(spx_sig_t *sw, int start, int end, int len, int *pitc
    for (i=0;i<N;i++)
    {
         best_score[i]=-1;
-        gain[i]=0;
         pitch[i]=start;
    }
 
@@ -112,11 +184,6 @@ void open_loop_nbest_pitch(spx_sig_t *sw, int start, int end, int len, int *pitc
       /* Update energy for next pitch*/
       energy[i-start+1] = energy[i-start] + SHR(MULT16_16(swn[-i-1],swn[-i-1]),6) - SHR(MULT16_16(swn[-i+len-1],swn[-i+len-1]),6);
    }
-   for (i=start;i<=end;i++)
-   {
-      corr[i-start]=0;
-      score[i-start]=0;
-   }
 
    pitch_xcorr(swn, swn-end, corr, len, end-start+1, stack);
 
@@ -134,14 +201,20 @@ void open_loop_nbest_pitch(spx_sig_t *sw, int start, int end, int len, int *pitc
          spx_word16_t g;
          spx_word32_t tmp;
          tmp = corr16[i-start];
-         if (SHR(corr16[i-start],4)>ener16[i-start])
-            tmp = SHL((spx_word32_t)ener16[i-start],14);
-         else if (-SHR(corr16[i-start],4)>ener16[i-start])
-            tmp = -SHL((spx_word32_t)ener16[i-start],14);
-         else
-            tmp = SHL(tmp,10);
-         g = DIV32_16(tmp, 8+ener16[i-start]);
-         score[i-start] = MULT16_16(corr16[i-start],g);
+         if (tmp>0)
+         {
+            if (SHR(corr16[i-start],4)>ener16[i-start])
+               tmp = SHL((spx_word32_t)ener16[i-start],14);
+            else if (-SHR(corr16[i-start],4)>ener16[i-start])
+               tmp = -SHL((spx_word32_t)ener16[i-start],14);
+            else
+               tmp = SHL(tmp,10);
+            g = DIV32_16(tmp, 8+ener16[i-start]);
+            score[i-start] = MULT16_16(corr16[i-start],g);
+         } else
+         {
+            score[i-start] = 1;
+         }
       }
    }
 #else
@@ -179,15 +252,18 @@ void open_loop_nbest_pitch(spx_sig_t *sw, int start, int end, int len, int *pitc
    }
 
    /* Compute open-loop gain */
-   for (j=0;j<N;j++)
+   if (gain)
    {
-      spx_word16_t g;
-      i=pitch[j];
-      g = DIV32(corr[i-start], 10+SHR(MULT16_16(spx_sqrt(e0),spx_sqrt(energy[i-start])),6));
-      /* FIXME: g = max(g,corr/energy) */
-      if (g<0)
-         g = 0;
-      gain[j]=g;
+       for (j=0;j<N;j++)
+       {
+          spx_word16_t g;
+          i=pitch[j];
+          g = DIV32(corr[i-start], 10+SHR(MULT16_16(spx_sqrt(e0),spx_sqrt(energy[i-start])),6));
+          /* FIXME: g = max(g,corr/energy) */
+                   if (g<0)
+                   g = 0;
+             gain[j]=g;
+       }
    }
 }
 
@@ -207,6 +283,7 @@ SpeexBits *bits,
 char *stack,
 spx_sig_t *exc2,
 spx_sig_t *r,
+spx_sig_t *new_target,
 int  *cdbk_index,
 int cdbk_offset
 )
@@ -232,7 +309,7 @@ int cdbk_offset
    x[0]=tmp;
    x[1]=tmp+nsf;
    x[2]=tmp+2*nsf;
-
+   
    e[0]=tmp2;
    e[1]=tmp2+nsf;
    e[2]=tmp2+2*nsf;
@@ -397,8 +474,11 @@ int cdbk_offset
    err=0;
    for (i=0;i<nsf;i++)
    {
-      spx_sig_t perr=target[i]-SHL((MULT16_32_Q15(SHL(gain[0],7),x[2][i])+MULT16_32_Q15(SHL(gain[1],7),x[1][i])+MULT16_32_Q15(SHL(gain[2],7),x[0][i])),2);
-      spx_word16_t perr2 = PSHR(perr,15);
+      spx_word16_t perr2;
+      spx_sig_t tmp = SHL((MULT16_32_Q15(SHL(gain[0],7),x[2][i])+MULT16_32_Q15(SHL(gain[1],7),x[1][i])+MULT16_32_Q15(SHL(gain[2],7),x[0][i])),2);
+      spx_sig_t perr=target[i]-tmp;
+      new_target[i] = target[i] - tmp;
+      perr2 = PSHR(perr,15);
       err = ADD64(err,MULT16_16(perr2,perr2));
       
    }
@@ -408,8 +488,11 @@ int cdbk_offset
    
    err=0;
    for (i=0;i<nsf;i++)
-      err+=(target[i]-gain[2]*x[0][i]-gain[1]*x[1][i]-gain[0]*x[2][i])
-      * (target[i]-gain[2]*x[0][i]-gain[1]*x[1][i]-gain[0]*x[2][i]);
+   {
+      spx_sig_t tmp = gain[2]*x[0][i]+gain[1]*x[1][i]+gain[0]*x[2][i];
+      new_target[i] = target[i] - tmp;
+      err+=new_target[i]*new_target[i];
+   }
 #endif
 
    return err;
@@ -440,23 +523,23 @@ int cdbk_offset
 {
    int i,j;
    int cdbk_index, pitch=0, best_gain_index=0;
-   spx_sig_t *best_exc;
+   spx_sig_t *best_exc, *new_target, *best_target;
    int best_pitch=0;
    spx_word64_t err, best_err=-1;
    int N;
    ltp_params *params;
    int *nbest;
-   spx_word16_t *gains;
 
    N=complexity;
    if (N>10)
       N=10;
+   if (N<1)
+      N=1;
 
    nbest=PUSH(stack, N, int);
-   gains = PUSH(stack, N, spx_word16_t);
    params = (ltp_params*) par;
 
-   if (N==0 || end<start)
+   if (end<start)
    {
       speex_bits_pack(bits, 0, params->pitch_bits);
       speex_bits_pack(bits, 0, params->gain_bits);
@@ -466,21 +549,25 @@ int cdbk_offset
    }
    
    best_exc=PUSH(stack,nsf, spx_sig_t);
+   new_target=PUSH(stack,nsf, spx_sig_t);
+   best_target=PUSH(stack,nsf, spx_sig_t);
    
    if (N>end-start+1)
       N=end-start+1;
-   open_loop_nbest_pitch(sw, start, end, nsf, nbest, gains, N, stack);
+   open_loop_nbest_pitch(sw, start, end, nsf, nbest, NULL, N, stack);
    for (i=0;i<N;i++)
    {
       pitch=nbest[i];
       for (j=0;j<nsf;j++)
          exc[j]=0;
       err=pitch_gain_search_3tap(target, ak, awk1, awk2, exc, par, pitch, p, nsf,
-                                 bits, stack, exc2, r, &cdbk_index, cdbk_offset);
+                                 bits, stack, exc2, r, new_target, &cdbk_index, cdbk_offset);
       if (err<best_err || best_err<0)
       {
          for (j=0;j<nsf;j++)
             best_exc[j]=exc[j];
+         for (j=0;j<nsf;j++)
+            best_target[j]=new_target[j];
          best_err=err;
          best_pitch=pitch;
          best_gain_index=cdbk_index;
@@ -493,6 +580,8 @@ int cdbk_offset
    /*printf ("encode pitch: %d %d\n", best_pitch, best_gain_index);*/
    for (i=0;i<nsf;i++)
       exc[i]=best_exc[i];
+   for (i=0;i<nsf;i++)
+      target[i]=best_target[i];
 
    return pitch;
 }
