@@ -52,7 +52,7 @@
 
 #define MAX_IN_SAMPLES 640
 
-const SpeexMode *speex_mode_list[SPEEX_NB_MODES] = {&speex_nb_mode, &speex_wb_mode, &speex_uwb_mode};
+const SpeexMode * const speex_mode_list[SPEEX_NB_MODES] = {&speex_nb_mode, &speex_wb_mode, &speex_uwb_mode};
 
 /* Extern declarations for all codebooks we use here */
 extern const signed char gain_cdbk_nb[];
@@ -375,7 +375,6 @@ static const SpeexNBMode nb_mode = {
    160,    /*frameSize*/
    40,     /*subframeSize*/
    10,     /*lpcSize*/
-   640,    /*bufSize*/
    17,     /*pitchStart*/
    144,    /*pitchEnd*/
 #ifdef FIXED_POINT
@@ -695,4 +694,13 @@ int speex_mode_query(const SpeexMode *mode, int request, void *ptr)
    return mode->query(mode->mode, request, ptr);
 }
 
+const SpeexMode * const speex_lib_get_mode (int mode)
+{
+#ifdef EPIC_48K
+  if (mode == SPEEX_MODEID_NB_48K) return &speex_nb_48k_mode;
+#endif
 
+  if (mode < 0 || mode > SPEEX_NB_MODES) return NULL;
+
+  return speex_mode_list[mode];
+}
