@@ -86,10 +86,6 @@
 #include <speex/speex_callbacks.h>
 #include "misc.h"
 
-#ifdef DISABLE_GLOBAL_POINTERS
-#include <speex/speex_noglobals.h>
-#endif
-
 #define MAX_FRAME_SIZE 2000
 
 #define readint(buf, base) (((buf[base+3]<<24)&0xff000000)| \
@@ -328,11 +324,7 @@ static void *process_header(ogg_packet *op, int enh_enabled, int *frame_size, in
    if (forceMode!=-1)
       modeID = forceMode;
 
-#ifdef DISABLE_GLOBAL_POINTERS
-   mode = speex_mode_new (modeID);
-#else
-   mode = speex_mode_list[modeID];
-#endif
+   mode = speex_lib_get_mode (modeID);
    
    if (header->speex_version_id > 1)
    {
@@ -557,7 +549,7 @@ int main(int argc, char **argv)
       outFile = "";
    wav_format = strlen(outFile)>=4 && (
                                        strcmp(outFile+strlen(outFile)-4,".wav")==0
-                                       || strcmp(inFile+strlen(inFile)-4,".WAV")==0);
+                                       || strcmp(outFile+strlen(outFile)-4,".WAV")==0);
    /*Open input file*/
    if (strcmp(inFile, "-")==0)
    {
